%% |peaks| Minimization with Genetic Algorithm
%  Copyright (c) 2010, The MathWorks, Inc.
%  All rights reserved.
%% Objective Function
% We wish find the minimum of the |peaks| function
clear all, close all, clc
peaks

%% Nonlinear Constraint Function
% Subject to a nonlinear constraint defined by a circular region of radius
% three around the origin
type circularConstraint

%% Define Optimization Problem
problem = createOptimProblem('fmincon',...
                             'objective',@(x) peaks(x(:,1),x(:,2)), ...
                             'nonlcon',@circularConstraint,...
                             'x0',[-1 -1],...
                             'lb',[-3 -3],...
                             'ub',[3 3],...
                             'options',optimset('OutputFcn',...
                                                @peaksPlotIterates))
                             
%% Run the solver |fmincon| from the inital point
% We can see the solution is not the global minimum
[x,f] = fmincon(problem)                                                    

%% Use Genetic Algorithm to Find the Global Minimum
% Solve the problem using ga.
problem.solver  = 'ga';
problem.fitnessfcn = problem.objective;
problem.nvars = 2;
problem.options = gaoptimset('PopInitRange',[-3;3],...
                             'OutputFcn',@peaksPlotIterates,...
                             'Display','iter')

[x,f] = ga(problem)