%% |peaks| Minimization with |GlobalSearch|
%  Copyright (c) 2010, The MathWorks, Inc.
%  All rights reserved.
%% Objective Function
% We wish find the minimum of the |peaks| function
clear all, close all, clc
peaks

%% Nonlinear Constraint Function
% Subject to a nonlinear constraint defined by a circular region of radius
% three around the origin
type circularConstraint

%% Define Optimization Problem
problem = createOptimProblem('fmincon',...
                             'objective',@(x) peaks(x(1),x(2)), ...
                             'nonlcon',@circularConstraint,...
                             'x0',[-1 -1],...
                             'lb',[-3 -3],...
                             'ub',[3 3],...
                             'options',optimset('OutputFcn',...
                                                @peaksPlotIterates))
                             
%% Run the solver |fmincon| from the initial point
% We can see the solution is not the global minimum
[x,f] = fmincon(problem)                                                    

%% Use |GlobalSearch| to Find the Global Minimum
% Define the globalsearch solver
close all
gs = GlobalSearch 
%% Run |GlobalSearch|
[x,f,exitflag,output,solutions] = run(gs, problem)