%% MultiStartDemo
% Date Reference:
% _Parameter estimation in nonlinear algebraic models via global
% optimization._ Computers & Chemical Engineering, Volume 22, Supplement 1,
% 15 March 1998, Pages S213-S220 William R. Esposito, Christodoulos A.
% Floudas

%  Copyright (c) 2010, The MathWorks, Inc.
%  All rights reserved.
close all, clear all, clc
%% Pharmacokinetic Data
t = [ 3.92,  7.93, 11.89, 23.90, 47.87, 71.91, 93.85, 117.84 ]              %#ok<*NOPTS>
c = [0.163, 0.679, 0.679, 0.388, 0.183, 0.125, 0.086, 0.0624 ]

plot(t,c,'o'), xlabel('t'), ylabel('c')

%% 3 Compartment Model
model = @(b,t) b(1)*exp(-b(4)*t) + b(2)*exp(-b(5)*t) + b(3)*exp(-b(6)*t)

%% Define Optimization Problem

problem = createOptimProblem('lsqcurvefit', ...
                             'objective', model, ...
                             'xdata', t, 'ydata', c, ...
                             'x0',ones(1,6),...
                             'lb', [-10 -10 -10  0   0   0 ],...
                             'ub', [ 10  10  10 0.5 0.5 0.5], ...
                             'options',optimset('OutputFcn',...
                             @curvefittingPlotIterates))
%% solve
b = lsqcurvefit(problem)                                                    

%% Multistart
ms = MultiStart                                                             
[b,fval,exitflag,output,solutions] = run(ms, problem, 50)                   %#ok<*NASGU,*ASGLU>

%%
curvefittingPlotIterates(solutions)

%%
problem.options.OutputFcn = {};
tic, [b,fval,exitflag,output,solutions] = run(ms, problem, 100), toc

%% Parallel Version
matlabpool open speedy-optim 16
ms.UseParallel = 'always'
tic, [bp,fvalp,exitflagp,outputp,solutionsp] = run(ms, problem, 100); toc
matlabpool close
 