function rastriginComparison(rfSelection)
close all
%  Copyright (c) 2010, The MathWorks, Inc.
%  All rights reserved.
%% Rastrigin's Function Comparison
% set option to 1 = standard rastrigin, 2 = scaled, 3 = scaled + shifted
if nargin == 0
    rfSelection = 1:3;
end

for case2run = rfSelection
    
%% Problem Definition
x0 = [2 1];     % starting point
minX = [0 0];   % global min
dom = [-6 6];
switch case2run
    case 1 % standard rastrigin's function
        rf = @(x)rastriginsfcn(x);
        rasLabel = 'Rastrigin''s Function';
    case 2 % scaled rastrigin's function
        rf = @(x)rastriginsfcn(x/10);
        rasLabel = 'Scaled Rastrigin''s Function';
        x0 = x0*10;
        dom = dom*10;
    case 3 % shifted rastrigin's function
        rf = @(x)rastriginsfcn((x-10)/10);
        rasLabel = 'Scaled and Shfited Rastrigin''s Function';
        x0 = x0*10 + 10;
        minX = minX*10 + 10;
        dom = dom*10 + 10;
end
figure,ezcontour(@(x,y) rf([x,y]),dom)
hold on, plot(minX(1),minX(2),'r*','MarkerSize',20)
plot(x0(1),x0(2),'k*','MarkerSize',20), hold off
axis equal tight
legend('FuncValue','Min','StartPt','Location','NorthWest')
drawnow
%% fminunc
tic
[xu fu flu ou] = fminunc(rf,x0)
tu = toc;
%% fminsearch
tic
[xs fs fls os] = fminsearch(rf,x0)
ts = toc;
%% GlobalSearch
problem = createOptimProblem('fmincon','objective',rf,'x0',x0,'lb',dom(1),'ub',dom(2));
gs = GlobalSearch;
tic
[xg fg flg og] = run(gs,problem)
tg = toc;
%% MultiStart
ms = MultiStart;
tic
problem = createOptimProblem('fminunc','objective',rf,'x0',x0,'lb',dom(1),'ub',dom(2));
[xm fm flm om] = run(ms,problem,100)
tm = toc;
%% PatternSearch
tic
[xp fp flp op] = patternsearch(rf,x0,[],[],[],[],dom(1),dom(2));
tp = toc;

%% Genetic Algorithm
initpop = 10*randn(20,2) + repmat([10 30],20,1);
opts = gaoptimset('InitialPopulation',initpop);
tic
[xga fga flga oga] = ga(rf,2,[],[],[],[],dom(1),dom(2))%,[],opts)
tga = toc;
%% Simulated Annealing
tic
[xsa fsa flsa osa] =simulannealbnd(rf,x0,dom(1),dom(2))
tsa = toc;

%% Show Tabular Results
rowhead = {'Solution-X1','Solution-X2','Objective','# Func Evals','Time'}';
colhead = {'fminunc','fminsearch','GlobalSearch','MultiStart',...
           'Pattern Search','Genetic Algorithm','Simulated Annealing'};
atab = {xu(1), xs(1), xg(1), xm(1), xp(1), xga(1), xsa(1);
        xu(2), xs(2), xg(2), xm(2), xp(2), xga(2), xsa(2);
        fu, fs, fg, fm, fp, fga, fsa;
        ou.funcCount, os.funcCount, og.funcCount, om.funcCount, op.funccount, oga.funccount, osa.funccount;
        tu, ts, tg, tm, tp, tga, tsa};
atab = [rowhead, atab];
atab = [{'Results',colhead{1:end}}; atab];
statdisptable(atab,'Comparison of Multiple Solver Runs',rasLabel)
set(gcf,'Position',[73 1010 926 138]);       

end
