function saGalacticTravelingSalesmanExample
% Ref: http://www.essex1.com/people/speer/starmodel.html

%  Copyright (c) 2010, The MathWorks, Inc.
%  All rights reserved.
clc; close all;
global xc yc zc N
global bestfval_log

% load star positions from 'stars.txt' 
stars = importdata('stars.txt');
N = size(stars.data, 1);
xc = stars.data(:,1); yc = stars.data(:,2); zc = stars.data(:,3); 
names = stars.rowheaders;
plot_stars(xc, yc, zc, names);
bestfval_log = [];

% set up schedule, and plot it
schedule = 1:N;
disp('Initial Schedule:')
disp(schedule);
plot_schedule(schedule);
title(sprintf('Initial Schedule: %s \n Distance = %f light year', ...
    num2str(schedule), cost_function(schedule)))
rotate3d
disp('Initial Total Distance:')
disp(cost_function(schedule));
pause

% set up options and call simulated annealing
rotate3d;
saopts = saoptimset('DataType', 'custom', ...
    'AnnealingFcn', @sa_anneal_fcn, ...
    'OutputFcns', @sa_output_fcn, ...
    'StallIterLimit', 100, ...
    'ReannealInterval', Inf, ...
    'InitialTemperature', 20);
[schedule,fval,exitFlag,output] = ...
    simulannealbnd(@cost_function, schedule, [], [], saopts); %#ok<*NASGU>

figure; plot(bestfval_log, '.-');
title 'Convergence for Galactic Traveling Salesman Problem';
xlabel 'Iteration'; ylabel 'Total Distance Traveled (light year)';

end

function d = cost_function(schedule)
global xc yc zc N
d = 0;
for i = 1:N-1
    d = d + ( ( xc(schedule(i)) - xc(schedule(i+1)) )^2 ...
        + ( yc(schedule(i)) - yc(schedule(i+1)) )^2 ...
        + ( zc(schedule(i)) - zc(schedule(i+1)) )^2) ^.5 ;
end
i = N;
d = d + ( ( xc(schedule(i)) - xc(schedule(1)) )^2 ...
    + ( yc(schedule(i)) - yc(schedule(1)) )^2 ...
    + ( zc(schedule(i)) - zc(schedule(1)) )^2) ^.5 ;
end

function schedule = sa_anneal_fcn(optimValues,problem) %#ok<*INUSD>
global N
schedule = optimValues.x;
T = fix(mean(optimValues.temperature)) + 1;
for i = 1:T
    idx = randi(N, 2, 1);
    while idx(1) == idx(2)
        idx = randi(N, 2, 1);
    end
    temp = schedule(idx(1));
    schedule(idx(1)) = schedule(idx(2));
    schedule(idx(2)) = temp;
end
end


function plot_stars(xc, yc, zc, names)
N = numel(xc);
f = figure('NumberTitle', 'off', ...
    'Name', 'Galactic Traveling Salesman Problem');
ax = axes('Color', [0 0 0]);
hold on;
textargs = {'VerticalAlignment', 'top', ...
    'HorizontalAlignment', 'center', ...
    'FontName', 'Gill Sans MT'};
plot3(xc(1), yc(1), zc(1), 'y*');
text(xc(1), yc(1), zc(1)-.5, names{1}, textargs{:}, 'Color', 'y');
for i = 2:N
    plot3(xc(i), yc(i), zc(i), 'w*');
    text(xc(i), yc(i), zc(i)-.5, names{i}, textargs{:}, 'Color', 'w');
end
% line([0 0], [0 0], [-15 +15], 'Color', 'g')
% line([-15 +15], [0 0], [0 0], 'Color', 'g')
% line([0 0], [-15 +15], [0 0], 'Color', 'g')
view(3); rotate3d;
end

function plot_schedule(schedule)
global xc yc zc
persistent l
if isempty(l) || ~ishghandle(l); l = plot3(0,0,0,'b-'); rotate3d; end
set(l, ...
    'XData', [xc(schedule); xc(schedule(1))], ...
    'YData', [yc(schedule); yc(schedule(1))], ...
    'ZData', [zc(schedule); zc(schedule(1))] );
title(sprintf('Best schedule found so far: %s \n Distance = %f light year', ...
    num2str(schedule), cost_function(schedule)))
end

function [stop,options,optchanged] = ...
    sa_output_fcn(options,optimValues,flag)
global bestfval_log
stop = false; optchanged = false;
if strcmpi(flag, 'init'); figure(1); end
fprintf('Iter=%d, Fval=%8.6f, Steps = %d\n', ...
    optimValues.iteration, optimValues.fval, ...
    fix(mean(optimValues.temperature))+1);
bestfval_log(end+1) = optimValues.bestfval;
plot_schedule(optimValues.x);
camorbit(1,0);
pause(.05);
end
