%% |peaks| Minimization with Simmulated Annealing
%  Copyright (c) 2010, The MathWorks, Inc.
%  All rights reserved.
%% Objective Function
% We wish find the minimum of the |peaks| function
clear all, close all, clc
peaks

%% Nonlinear Constraint Function
% Subject to a nonlinear constraint defined by a circular region of radius
% three around the origin
type circularConstraint

%% Define Optimization Problem
problem = createOptimProblem('fmincon',...
                             'objective',@(x) peaks(x(1),x(2)), ...
                             'nonlcon',@circularConstraint,...
                             'x0',[-1 -1],...
                             'lb',[-3 -3],...
                             'ub',[3 3],...
                             'options',optimset('OutputFcn',...
                                                @peaksPlotIterates))
                             
%% Run the solver |fmincon| from the inital point
% We can see the solution is not the global minimum
[x,f] = fmincon(problem)                                                    

%% Use Simmulated Annealing to Find the Global Minimum
% Solve the problem using simmulated annealing.  Note that simmulated
% annealing does not support nonlinear so we need to account for this in
% the objective function.
problem.solver  = 'simulannealbnd';
problem.objective = @(x) peaks(x(1),x(2)) + (x(1)^2 + x(2)^2 - 9);
problem.options = saoptimset('OutputFcn',@peaksPlotIterates,...
                             'Display','iter',...
                             'InitialTemperature',10,...
                             'MaxIter',300)

[x,f] = simulannealbnd(problem) 
f = peaks(x(1),x(2))